@@ -247,8 +247,7 @@ class InstallWizard(QDialog):
               +_("Leave these fields empty if you want to disable encryption.")
         from password_dialog import make_password_dialog, run_password_dialog
         self.set_layout( make_password_dialog(self, wallet, msg) )
-
-        run_password_dialog(self, wallet, self)
+        return run_password_dialog(self, wallet, self)
 
 
     def run(self):
@@ -269,13 +268,14 @@ class InstallWizard(QDialog):
                 return
             if not self.verify_seed(wallet):
                 return
+            ok, old_password, password = self.password_dialog(wallet)
             def create():
-                wallet.save_seed()
+                wallet.save_seed(password)
                 wallet.synchronize()  # generate first addresses offline
             self.waiting_dialog(create)
 
+
         elif action == 'restore':
-            # ask for seed and gap.
             seed = self.seed_dialog()
             if not seed:
                 return
@@ -287,10 +287,11 @@ class InstallWizard(QDialog):
                 QMessageBox.warning(None, _('Error'), _('Incorrect seed'), _('OK'))
                 return
 
-            wallet.save_seed()
+            ok, old_password, password = self.password_dialog(wallet)
+            wallet.save_seed(password)
+
 
         elif action == 'watching':
-            # ask for seed and gap.
             mpk = self.mpk_dialog()
             if not mpk:
                 return
@@ -301,7 +302,11 @@ class InstallWizard(QDialog):
                 
         #if not self.config.get('server'):
         if self.network:
-            self.network_dialog()
+            if self.network.interfaces:
+                self.network_dialog()
+            else:
+                QMessageBox.information(None, _('Warning'), _('You are offline'), _('OK'))
+                self.network.stop()
 
         # start wallet threads
         wallet.start_threads(self.network)
@@ -318,6 +323,4 @@ class InstallWizard(QDialog):
             else:
                 QMessageBox.information(None, _('Information'), _("This wallet was restored offline. It may contain more addresses than displayed."), _('OK'))
 
-        self.password_dialog(wallet)
-
         return wallet
